## Exercise 2: Bare Metal Simulation
In this exercise, we will simulate a simple bare metal system. The system will consist of a CPU,
memory, and a UART. The CPU will execute a simple program that writes a message to the UART. The
UART will transmit the message to the host computer. The host computer will receive the message and
display it on the terminal.

- Install the qemu-system-arm package: The qemu-system-arm package is a fast processor emulator
using dynamic translation to achieve good emulation speed. It is used to emulate the ARM
architecture. To install it, run the following command:
```bash
sudo apt install qemu-system-arm
```
- Install the gcc-arm-none-eabi package: The gcc-arm-none-eabi package is a cross-compiler for the
ARM architecture. It is used to compile code for the ARM architecture on a different architecture.
```bash
sudo apt install gcc-arm-none-eabi
```

### Step 1: Create a main.c file with the following content:
```c 
volatile unsigned int *const USART1_PTR = (unsigned int *)0x40011004;

void my_printf(const char *s) {
    while(*s != '\0') { /* Loop until end of string */
        *USART1_PTR= (unsigned int)(*s); /* Transmit char */
        s++; /* Next char */
    }
}

int main(void) {
        my_printf("Hello world!\n");
}

```
### Create a linker script file named `linker.ld` with the following content:
```ld
/* ENTRY directive defines the entry point of the program */
ENTRY(_start)

/* SECTIONS directive defines the memory regions of the program */
SECTIONS
{
    /* text section (code)*/
    .text : { *(.text*) } 		
    /* data section, initialized variables */
    .data : { *(.data) }
    /* bss section, uninitialized variables */
    .bss : { *(.bss*) }
    /* stack section 
    Define the stack to the last RAM address of the NetduinoPlus2 board */
    stack_top = 0x2001ffff; 
}

```

### Create a startup file named `startup.s` with the following content:
```asm
.word stack_top     // Address of the stack_top
.word Init_Function // Address of the Init_Function

// The thumb_func is used to make sure the function is in thumb mode,
// which is required for the Cortex-M0+.
.thumb_func 
.global _start

_start:
    BL main
    B .

```

### Compile the program using the following command:
```bash
arm-none-eabi-as -mcpu=cortex-m4 -g startup.s -o startup.o
```

The -g flag is used to generate debugging information.
Now, compile the main.c file using the following command:
```bash
arm-none-eabi-gcc -c -mcpu=cortex-m4 -mthumb -g main.c -o main.o
```

The -mcpu flag specifies the target ARM architecture. The -mthumb flag specifies to use the Thumb
instruction set. The -c flag specifies to compile the source file without linking.

```bash
arm-none-eabi-ld -T linker.ld main.o startup.o -o main.elf
```
The -T flag specifies the linker script file. The main.o and startup.o files are the object files
generated by the compiler.

### Run the program using QEMU:
```bash
qemu-system-arm -M netduino2 -nographic -kernel main.elf
```
The -M flag specifies the machine type. The -nographic flag specifies to run QEMU without a
graphical interface. The -kernel flag specifies the kernel image to run. In this case it is the
main.elf file.

You should see the message "Hello world!" printed on the terminal in an infinite loop. for endin the
simulation press `Ctrl + A` and then `X`.

